#############################################################################
#	base.cc	Produced by Bidl
#############################################################################
#

import sys;

sys.path.extend(['/usr/beam/lib']);

from Beam import *;

import libBase;

# BObjDInfo Object Implementation
class BObjDInfo(BObjData):
	def __init__(s):
		BObjData.__init__(s);
		s.type = 0;
		s.description = BObjString();

# BObjDTodo Object Implementation
class BObjDTodo(BObjData):
	def __init__(s):
		BObjData.__init__(s);
		s.type = 0;
		s.priority = BObjInt();
		s.done = BObjInt();
		s.startDate = BObjDate();
		s.endDate = BObjDate();
		s.description = BObjString();

# BObjDAddress Object Implementation
class BObjDAddress(BObjData):
	def __init__(s):
		BObjData.__init__(s);
		s.type = 0;
		s.firstName = BObjString();
		s.lastName = BObjString();
		s.company = BObjString();
		s.address = BObjString();
		s.postCode = BObjString();
		s.tel = BObjString();
		s.mobile = BObjString();
		s.fax = BObjString();
		s.email = BObjString();
		s.web = BObjString();
		s.groups = BObjString();
		s.christmas = BObjString();
		s.misc = BObjString();
		s.telWork = BObjString();

# BObjDTest Object Implementation
class BObjDTest(BObjData):
	def __init__(s):
		BObjData.__init__(s);
		s.type = 0;
		s.test1 = BObjString();
		s.test2 = BObjString();

# BObjDWhatsOn Object Implementation
class BObjDWhatsOn(BObjData):
	def __init__(s):
		BObjData.__init__(s);
		s.type = 0;
		s.date = BObjDate();
		s.dateLast = BObjDate();
		s.tcGroup = BObjString();
		s.startTime = BObjString();
		s.startPlace = BObjString();
		s.rideLength = BObjString();
		s.ease = BObjString();
		s.description = BObjString();
		s.who = BObjString();
		s.tel = BObjString();
		s.email = BObjString();
		s.web = BObjString();

# BObjDMttest Object Implementation
class BObjDMttest(BObjData):
	def __init__(s):
		BObjData.__init__(s);
		s.type = 0;
		s.field1 = BObjString();
		s.field2 = BObjString();

# Item Object Implementation
class Item(BObjData):
	def __init__(s):
		BObjData.__init__(s);
		s.type = 0;
		s.qty = BObjInt();
		s.info = BObjString();

# Despatch Object Implementation
class Despatch(BObjData):
	def __init__(s):
		BObjData.__init__(s);
		s.type = 0;
		s.start = BObjInt();
		s.item1 = Item();
		s.item2 = Item();

