/*******************************************************************************
 *	BUrl.h		BEAM BUrl access class
 *			T.Barnaby,	BEAM Ltd,	12/11/02
 *	Copyright (c) 2012 All Right Reserved, Beam Ltd, http://www.beam.ltd.uk
 *******************************************************************************
 */
#ifndef BURL_H
#define BURL_H	1

#include	<stdio.h>
#include	<BString.h>
#include	<BError.h>

/// Basic access to a Url 
class BUrl {
public:
			BUrl();
			~BUrl();

	BError		readString(BString url, BString& str);					///< Reads URL
private:
	static int	oinit;
	static size_t	writeData(void* data, size_t size, size_t elSize, void* stream);
	
	BString		ores;
};

#endif
