/*******************************************************************************
 *	PupeGeog.h	Pupe Bus Slot Location class
 *			T.Barnaby,	BEAM Ltd,	2007-09-16
 *******************************************************************************
 */
#ifndef PupeGeog_H
#define PupeGeog_H	1

#include <BError.h>
#include <BArray.h>

class PupeGeog {
public:
	enum		{ MaxNumBoards = 16 };

			PupeGeog();

	BError		scan(BArray<int> devs);
	int		getBoardId(unsigned int slot);
private:
	int		oboards[MaxNumBoards];
};

#endif
