/* Title:	tmsControlGui.h 
* Author:	M.Thomas BEAM Ltd
* Date:		2007-02-13
*
* Contents:	
*
* Mod Rec:
*
*/


#ifndef TMSCONTROLGUI_H
#define TMSCONTROLGUI_H
#include <BString.h>
#include <Config.h>


extern BString 	ghostName;
extern int	gring;
extern Config	gconfig;


#endif
