/*******************************************************************************
 *	Gui.cc		General Gui Interface
 *			T.Barnaby,	BEAM Ltd,	2005-09-29
 *******************************************************************************
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <Gui.h>

Gui::Gui(Control& control) :
	ocontrol(control){
}

Gui::~Gui(){
}
			
BError Gui::init(int argc, char** argv){
	BError		err;
	char		banner[256];

	sprintf(banner,"tmsControlGui on %s connected to: %s ring: %d", BSocketAddressINET::getHostName().retStr(), ghostName.retStr(), gring);
	oapp = new QApplication(argc,argv);
	oappGui = new TmsControlMainWin(ocontrol);
	oapp->setMainWidget(oappGui);
	oappGui->init(banner);
	oappGui->show();
	return err;
}

void Gui::run(){
	oapp->exec();
}
