body {
      	font: 			80% Verdana, Helvetica, Arial, sans-serif;
	background:	 	#f0f0f0;
    	color: 			Black;
}

table {
      	font: 			100% Verdana, Helvetica, Arial, sans-serif;
}

a {
	text-decoration: 	none;
	color: 			#436976;
	background-color: 	transparent;
}

a:hover {
	text-decoration: 	underline;
}

div.tabs {
    background: 		#006486;
    border-collapse: 		collapse;
    border-bottom-color: 	#DEE7EC;
    border-bottom-style: 	solid;
    border-bottom-width: 	4px;
    padding: 			0.5em 0em 0em 2em; 
    white-space: 		nowrap;
}

div.tabs a {
    background: 	#006486;
    border-color: 	#8CACBB;
    border-width: 	1px; 
    border-style: 	solid solid none solid;
    color: 		#ffffff;
    font-weight: 	normal;
    height: 		1.2em;
    margin-right: 	0.5em;
    padding: 		0em 1em;
}

div.tabs a.selected {
    background: 	#DEE7EC;
    border: 		1px solid 	#8CACBB;
    border-bottom: 	#DEE7EC 1px solid;
    color: 		#436976;
    font-weight: 	normal;
}

div.tabs .active {
    background: 	#DEE7EC;
    border: 		1px solid #8CACBB;
    border-bottom: 	1px solid #DEE7EC;
    color: 		#436976;
    font-weight: 	normal;
}


div.tabs a:hover {
    background: 	#DEE7EC;
    border-color: 	#8CACBB;
    border-bottom-color: #DEE7EC;
    color: 		#436976;
}

.footer {
    font: 		100% Verdana, Helvetica, Arial, sans-serif;
    background: 	#DEE7EC;
    border-top: 	1px solid #8CACBB;
    border-bottom: 	1px solid #8CACBB;
    clear: 		both;
    float: 		none;
    padding: 		0.1em 0.1em 0.1em 5.1em;
    text-align: 	left;
    font-weight:	bold;
}

.leftSide {
   vertical-align:	top;
   width:		150px;
   background-color:	#f0f0f0;
}

.centre {
   vertical-align:	top;
   background-color: 	#ffffff;
   padding: 	      	0.5em 0em 1em 1em;
}

.beam_title {
   background-color:  #f0f0f0;
}






.even {
    background-color: 	#F7F9FA;
}

.odd {
    background-color: 	transparent;
}

.highlight {
    background-color: 	#F7F9FA;
}

.error {
	color: 			#FF0000;
}

.editError {
	color: 			#FF0000;
}

.viewList th {
	color:	#436976;
}
