#!/bin/sh
################################################################################
#	TmsWeb	This shell script takes care of starting and stopping
#			the TmsWeb daemons
################################################################################
#
# chkconfig: 2345 99 20
# description: TmsWeb is the startup for TmsWeb server daemons
# processname: TmsWeb

# Source function library.
. /etc/rc.d/init.d/functions

# Source networking configuration.
. /etc/sysconfig/network

export PATH=/usr/tms/bin:$PATH

# See how we were called.
case "$1" in
  start)
        # Start daemons.
        echo -n "Starting TmsWeb System"
	daemon /usr/tms/bin/tmsWeb
        echo
        ;;
  stop)
        # Stop daemons.
        echo -n "Shutting down TmsWeb System"
	killproc tmsWeb
        echo
        ;;
  status)
	status tmsWeb
	;;
  restart|reload)
	$0 stop
	$0 start
	;;
  *)
        echo "Usage: tmsWeb {start|stop|restart|reload|status}"
        exit 1
esac

exit 0
