/*******************************************************************************
 *	BoapnsD.h	Produced by Bidl
 *	Copyright (c) 2012 All Right Reserved, Beam Ltd, http://www.beam.ltd.uk
 *******************************************************************************
 */

#ifndef BOAPNSD_H
#define BOAPNSD_H 1

#include <Boap.h>
#include <BObj.h>
#include <BDate.h>
#include <BTimeStamp.h>
#include <BComplex.h>
#include <BList.h>
#include <BArray.h>

namespace Boapns {
	class BoapEntry {
	public:
		BoapEntry(BString pname = BString(), BString phostName = BString(), BList<BString > paddressList = BList<BString >(), BUInt32 pport = BUInt32(), BUInt32 pservice = BUInt32());
	public:
		BString	name;
		BString	hostName;
		BList<BString >	addressList;
		BUInt32	port;
		BUInt32	service;
	};

}

#endif
